var ZeedhiFields = angular.module("ZeedhiFields", []);

ZeedhiFields.directive("zhImageUpload", function() {
	return function(scope, elementContainer, attrs) {

		var element = elementContainer.find("input");

		var formElement = elementContainer.parent();

		var imageElement = formElement.find("img");

		var domElement = element[0];

		element.on("change", function() {
			if (domElement.files && domElement.files[0]) {
				scope.files = domElement.files;
				var reader = new FileReader();
				reader.onload = function(e) {
					imageElement.attr('src', e.target.result);
					scope.fieldRow = e.target.result;
				};
				reader.readAsDataURL(domElement.files[0]);
			} else {
				if (scope.fieldRow) {
					imageElement.attr('src', scope.fieldRow);
				} else {
					imageElement.attr('src', '');
				}
			}
		});
	};
});

ZeedhiFields.directive("zhInputSearch", ['$compile', '$timeout',
	function($compile, $timeout) {
		return {
			restrict: 'C',
			require: 'ngModel',
			scope: {},
			link: function(scope, element, attrs, ctrl) {
				// limit to input element of specific types
				var inputTypes = /text|search|tel|url|email|password|number/i;
				if (element[0].nodeName !== "INPUT") {
					throw new Error("zhInputSearch is limited to input elements");
				}
				if (!inputTypes.test(attrs.type)) {
					throw new Error("Invalid input type for zhInputSearch: " + attrs.type);
				}

				// compiled reset icon template
				var clearButton = $compile(
					'<span ng-show="enabled" class="clear-button" zh-touchstart="reset()"><svg><polygon fill="#B75520" points="14,1.243 12.756,0 7,5.756 1.244,0 0,1.243 5.756,7 0,12.756 1.244,14 7,8.244 12.756,14 14,12.756 8.243,7" /></svg></span>'
				)(scope);
				element.after(clearButton);

				element.on('input', function() {
					scope.enabled = !ctrl.$isEmpty(element.val());
				});

				scope.reset = function() {
					ctrl.$setViewValue("");
					ctrl.$render();
					scope.enabled = false;
					$timeout(function() {
						element[0].focus();
					}, 0, false);
				};
			}
		};
	}
]);

ZeedhiFields.directive("zhInputNumber", function(templateManager) {
	return {
		require: 'ngModel',
		link: function($scope, element, attrs, controller) {
			var input = element;

			var minValue, maxValue;

			function updateMinMaxValue() {
				minValue = parseFloat($scope.field.minValue);
				minValue = isNaN(minValue) ? -Number.MAX_VALUE : minValue;

				maxValue = parseFloat($scope.field.maxValue);
				maxValue = isNaN(maxValue) ? Number.MAX_VALUE : maxValue;
			}

			var currentVal;

			$scope.buttonClick = function(plus) {
				updateMinMaxValue();

				var val = parseInt($scope.fieldRow, 10);
				if (isNaN(val)) {
					val = 0;
				}

				if (!plus) {
					if (val > minValue) {
						input.val(val - 1).change();
					}
				} else {
					if (val < maxValue) {
						input.val(val + 1).change();
					}
				}
			};

			var blockButton = function(value) {
				if(!$scope.field.readOnly){
					var divContainer = element.parent().parent();
					var minusButton = divContainer.children().eq(1);
					var plusButton = divContainer.children().eq(3);
					minusButton.attr('disabled', value <= minValue);
					plusButton.attr('disabled', value >= maxValue);
				}
			};

			function inputChange(event, value) {
				updateMinMaxValue();

				var val = Util.isUndefinedOrNull(value) ? input.val() : value;
				if (Util.isEmptyOrBlank(val)) {
					currentVal = val;
				} else {
					var parsedValue = parseFloat(val); //Parse value to accept strings with number content
					if (!isNaN(parsedValue) && (parsedValue >= minValue && parsedValue <= maxValue)) {
						currentVal = parsedValue;
					} else if(isNaN(val) && typeof val !== "number"){ //If value is an string with text content
						currentVal = val;
					}
				}
				input.val(currentVal);
				blockButton(currentVal);
				return currentVal;
			}

			function keydownEvent(e) {
				var value = input.val();

				var commaOrPoint = (e.keyCode == 188 || e.keyCode == 190) && (~value.indexOf(",") || ~value.indexOf("."));
				var checkExp = e.keyCode == 69 && (~value.indexOf("e") || ~value.indexOf("E"));
				if (commaOrPoint || checkExp) {
					e.preventDefault();
				}
			}

			input.on('keydown', keydownEvent);
			input.on('change', inputChange);
			var value = parseFloat($scope.field.value());

			$scope.fieldRow = isNaN(value) ? null : value;

			controller.$formatters.push(function(value) {
				inputChange(undefined, value);
				return currentVal;
			});
		}
	};
});

ZeedhiFields.directive("zhNumberKeyboard", function($document, templateManager, ZhUtil, maskEngine, $timeout) {

	return function(scope, element, attrs) {

		var input = element.find("input");
		var keyboard = element.find(".zh-number-keyboard");
		var currentVal;
		var params = scope.field.mask && scope.field.mask.params ? scope.field.mask.params : {};
		var precision = params.precision || 2;
		var decimal = params.decimal || $.currency.i18n[ZhUtil.getLang()].decimal;
		var minValue = parseFloat(scope.field.minValue);
		var maxValue = parseFloat(scope.field.maxValue);
		minValue = isNaN(minValue) ? -Number.MAX_VALUE : minValue;
		maxValue = isNaN(maxValue) ? Number.MAX_VALUE : maxValue;

		if (scope.field.maxlength && !scope.field.maxValue) {
			maxValue = "0";
			for (var i = 0; i < scope.field.maxlength; i++) {
				maxValue += "9";
			}
			maxValue = parseInt(maxValue);
		}

		var hasMask = function(field) {
			return !!field.mask;
		};

		var hasTypeOfMask = function(field, name) {
			return field.mask && field.mask.type === name;
		};

		function getLocaleSeparator() {
			var num = 1.1;
			return num.toLocaleString().slice(1, -1);
		}

		scope.localeSeparator = scope.field.separator || getLocaleSeparator();
		if (hasTypeOfMask(scope.field, "currency")) {
			scope.localeSeparator = decimal;
		}

		scope.regSeparator = scope.localeSeparator === '.' ? "\\." : scope.localeSeparator;
		scope.regSeparator = new RegExp(scope.regSeparator, "g");


		var blockButton = function(value) {
			if (!hasMask(scope.field) || scope.field.readOnly) {
				var minusButton = element.find('.spin-button-minus');
				var plusButton = element.find('.spin-button-plus');
				var val = value ? value.toString().replace(scope.regSeparator, '.') : "";
				minusButton.attr('disabled', (val && parseFloat(val) <= minValue) || scope.field.readOnly);
				plusButton.attr('disabled', (val && parseFloat(val) >= maxValue) || scope.field.readOnly);
			}
		};

		scope.increase = function(val) {
			scope.result = scope.fieldRow || "";
			scope.result += val;
			if (validate(scope.result) || (minValue > 0 && scope.result <= maxValue) || (maxValue <= 0 && scope.result >= minValue)) {
				scope.fieldRow = scope.result;
				if (hasTypeOfMask(scope.field, "fix")) {
					scope.fieldRow = Util.applyMask(scope.result, scope.field.mask, maskEngine, ZhUtil);
				} else {
					blockButton(scope.result);
				}
			} else {
				scope.result = scope.result.substr(0, scope.result.length - 1);
			}
		};

		var validate = function(value) {
			var val = value.replace(scope.regSeparator, '.');
			var precisionLength = true;
			if (hasTypeOfMask(scope.field, "currency") && value.indexOf(decimal) !== -1) {
				precisionLength = value.slice(value.indexOf(decimal) + 1).length <= precision;
			}
			return (val <= maxValue && val >= minValue && precisionLength) || hasTypeOfMask(scope.field, "fix");
		};

		scope.toggleSignal = function() {
			var change;
			if (scope.fieldRow !== '0' && scope.fieldRow !== '') {
				change = scope.fieldRow[0] === '-' ? scope.fieldRow.slice(1) : '-' + scope.fieldRow;
				if (validate(change)) {
					scope.result = scope.fieldRow = change;
					blockButton(scope.fieldRow);
				}
			}
		};

		scope.backspace = function() {
			if (scope.fieldRow) {
				var number = scope.fieldRow.toString().slice(0, -1);
				scope.result = scope.fieldRow = number || "";
				blockButton(number);
			}
		};

		scope.clearField = function() {
			scope.fieldRow = "";
		};

		scope.confirm = function() {
			keyboard.toggle();
			input.blur();
			if (scope.field.onConfirm) {
				scope.field.onConfirm();
			}
		};

		input.focus(function() {
			input.blur();
		});

		input.click(function(evt) {
			var label = element.find("label");

			evt.stopPropagation();

			$(".zh-number-keyboard:visible").hide().parent().find('input').blur();
			keyboard.toggle();

			var keys = $.makeArray(keyboard.find("td"));
			var keySize = parseInt(window.getComputedStyle(keyboard[0]).width, 10) / 4;
			var inputHeight = parseInt(window.getComputedStyle(input[0]).height, 10);
			var inputWidth = parseInt(window.getComputedStyle(input[0]).width, 10);
			var arrowHeight = 10;
			var labelHeight = 0;

			if (label) {
				var h = parseInt(window.getComputedStyle(label[0]).height, 10);
				if (!isNaN(h)) {
					labelHeight = h;
				}
			}

			// Altura do teclado = input + seta + 4 tr's
			var keyboardHeight = inputHeight + arrowHeight + (keySize * 5);

			// Ajusta o keySize de acordo com o ambiente mobile ou desktop
			if (keyboardHeight > window.innerHeight) {
				keySize = (window.innerHeight - inputHeight - arrowHeight) / 4 - 10;
			}

			for (var i = 0; i < keys.length; i++) {
				keys[i].style.width = (keySize) + "px";
				keys[i].style.height = (keySize) + "px";
			}

			// Posiciona verticalmente o teclado de acordo com o inputHeight e a altura da seta
			keyboard[0].style.top = inputHeight + arrowHeight + labelHeight + 5 + "px";
			var keyboardWidth = parseInt(window.getComputedStyle(keyboard[0]).width, 10);

			var left = (inputWidth - keyboardWidth) / 2;

			if ($(keyboard).offset().left + keyboardWidth > window.innerWidth) {
				keyboard[0].style.right = "0px";
				keyboard[0].classList.add('right-pos-arrow');
			} else if (left < 0) {
				keyboard[0].style.left = "0px";
				keyboard[0].classList.add('left-pos-arrow');
			}


			// Evita que to teclado apareça quando o input está ativado
			input.blur();

			var keyboardOffset = keyboard.offset();
			var scroller = keyboard.closest("[zh-scroller]");
			var top = keyboardOffset.top - scroller.offset().top;
			//Need to wait the keyboar be hided
			setTimeout(function() {
				if (window.innerHeight - input.offset().top + inputHeight < keyboardHeight) {
					scroller.animate({
						scrollTop: top + inputHeight
					}, 500);
				}
			}, 500);

		});
		var hideKeyboard = function() {
			if (keyboard.is(':visible')) {
				keyboard.hide();
				input.blur();
				input.change();
			}
		};

		$document.click(hideKeyboard);
		scope.$on('$destroy', function() {
			$document.unbind('click', hideKeyboard);
		});

		scope.buttonClick = function(plus) {
			if (!hasMask(scope.field)) {
				var val = scope.fieldRow ? scope.fieldRow.replace(scope.regSeparator, '.') : "";
				if (isNaN(val) || val === "") {
					val = "0";
				}
				val = plus ? (parseFloat(val) + 1).toString() : (parseFloat(val) - 1).toString();
				if (scope.localeSeparator !== '.') {
					val = val.replace(/\./g, scope.localeSeparator);
				}
				if (validate(val)) {
					input.val(val);
					scope.result = val;
				}
				input.change();
			}
		};

		function inputChange() {
			if (!hasTypeOfMask(scope.field, "fix")) {
				var val = scope.result && validate(scope.result) ? scope.result : "";
				if (!isNaN(val.replace(scope.regSeparator, '.')) && validate(val)) {
					currentVal = val;
				}
				if (hasMask(scope.field)) {
					val = Util.applyMask(val, scope.field.mask, maskEngine, ZhUtil);
				}
				scope.fieldRow = currentVal || "";
				blockButton(scope.fieldRow);
			}
		}

		input.on('change', inputChange);

		$timeout(function() {
			blockButton();
		}, 100, false);

	};
});

ZeedhiFields.directive("zhInputPhone", function(templateManager) {

	var DIGIT_0 = 48;
	var DIGIT_9 = 57;
	var NUM_PAD_0 = 96;
	var NUM_PAD_9 = 105;

	return {
		require: 'ngModel',
		link: function($scope, element, attrs, controller) {
			var valueChange = function(e) {
				var key = e.charCode || e.keyCode || 0;
				if(e.ctrlKey && e.key.toUpperCase() === 'V'){
					return true;
				}

				if (e.ctrlKey || e.altKey || e.shiftKey || e.metaKey) {
					return false;
				}
				// Reset if they highlight and type over first char.
				else if (element.val().charAt(0) !== '(') {
					element.val('(');
				}
				if (key == 8 && element.val().length == 15) {
					element.val([element.val().slice(0, 9), "-", element.val().charAt(9) + element.val().slice(11)].join(''));
				}
				// Auto-format- do not expose the mask as the user begins to type
				if ((e.keyCode >= DIGIT_0 && e.keyCode <= DIGIT_9) || (e.keyCode >= NUM_PAD_0 && e.keyCode <= NUM_PAD_9)) {
					if (element.val().length === 3) {
						element.val(element.val() + ')');
					}
					if (element.val().length === 4) {
						element.val(element.val() + ' ');
					}
					if (element.val().length === 9) {
						element.val(element.val() + '-');
					} else
					if (element.val().length == 14) {
						if (element.val().charAt(10) != "-") {


							element.val([element.val().slice(0, 9) + element.val().charAt(10), "-", element.val().slice(11)].join(''));
						}
					}
				}

				// Allow numeric (and tab, backspace, delete) keys only
				var isNumberOnNumPad = e.originalEvent && e.originalEvent.location === 3 && (key >= NUM_PAD_0 && key <= NUM_PAD_9);

				return (key == 8 ||
						key == 9 ||
						key == 46 ||
						(key >= DIGIT_0 && key <= DIGIT_9) ||
						isNumberOnNumPad);
			};

			var pasteHandler = function (e) {
				var pastedValue = e.originalEvent.clipboardData.getData('text');

				for (var i = 0; i < pastedValue.length; i++) {
					var newEvent = {
						charCode: pastedValue[i].charCodeAt(),
						keyCode: pastedValue[i].charCodeAt(),
						ctrlKey: false,
						altKey: false,
						shiftKey: false,
						metaKey: false,
						key: pastedValue[i]
					};

					if(valueChange(newEvent) && element.val().length < 15){
						var currentFieldValue = element.val();
						element.val(currentFieldValue + pastedValue[i]);
					}
				}

				return false;
			};

			element.bind('keydown', valueChange);
			element.bind('paste', pasteHandler);

			var oldValue = "";
			var changed = false;

			element.bind('change', function() {
				changed = true;
			});
			element.bind('focus click', function() {
				oldValue = element.val();
				changed = false;
				var val = element.val();
				element.val('').val(val); // Ensure cursor remains at the end
			});
			element.bind('blur', function() {

				if (element.val() === '(') {
					element.val('');
				}
				if (oldValue !== element.val() && !changed) {
					element.trigger('change');
				}
			});
		}
	};
});

ZeedhiFields.directive('zhContenteditable', function() {
	return {
		link: function($scope, element, attrs) {
			if (Util.isDesktop()) {
				$scope.changeContentEditable = function() {
					element.attr("contenteditable", attrs.zhContenteditable !== "" ? attrs.zhContenteditable : "true");
					element.attr("tabindex", element.attr("contenteditable") === "true" ? "0" : "-1"); 
					element.children().attr("contenteditable", false);
				};
				
				attrs.$observe('zhContenteditable', function() {
					$scope.changeContentEditable();
				});

				element.on('keydown', function(e) {
					return e.ctrlKey || e.keyCode == 9 || e.keyCode == 13 || e.keyCode == 27 || (e.keyCode >= 112 && e.keyCode <= 126); // TAB, ENTER, ESC, f? key
				});
			}
		}
	};
});

ZeedhiFields.directive('zhFieldSelect', function(templateManager) {
	return {
		restrict: 'C',
		link: function($scope, element) {
			var vField = $scope.field.valueField;

			function setOutDataFromFieldValue(value) {
				if (value) {
					var matchRows = $scope.field.dataSource.data.filter(function(item) {
						return item[vField] == value;
					});
					if (matchRows.length > 0) {
						Object.keys($scope.field.outData).forEach(function(key) {
							$scope.field.widget.currentRow[$scope.field.outData[key]] = matchRows[0][key];
						});
					}
				}
			}

			if ($scope.field.outData) {
				$scope.$watch(function() {
					return $scope.field.widget.currentRow[$scope.field.outData[vField]];
				}, function(value) {
					setOutDataFromFieldValue(value);
				});
			}
		}
	};
});

ZeedhiFields.directive('zhCheckbox', function(templateManager) {
	return {
		link: function($scope, $element) {
			var field = $scope.field;
			var input = $element.find("input");
			var label = $element.find("label");

			field.valueToTrue = Util.isUndefined(field.valueToTrue) ? "Y" : field.valueToTrue;
			field.valueToFalse = Util.isUndefined(field.valueToFalse) ? "N" : field.valueToFalse;

			function toggleValue() {
				if (!input.is(':disabled')) {
					field.checked = !field.checked;

					if (!field.readOnly) {
						$scope.widget.currentRow[field.name] = field.checked ? field.valueToTrue : field.valueToFalse;
						$scope.widget.currentRow.changed = true;

						if (field.change) {
							field.change();
						}

						templateManager.onUpdate();
					}

				}
			}

			var handleCheckedState = function() {
				if ($scope.widget.currentRow[field.name] === field.valueToTrue) {
					field.checked = true;
				} else if ($scope.widget.currentRow[field.name] === field.valueToFalse) {
					field.checked = false;
				} else if (field.checked) {
					$scope.widget.currentRow[field.name] = field.valueToTrue;
				} else {
					field.checked = false;
					$scope.widget.currentRow[field.name] = field.valueToFalse;
				}
			};

			$scope.field.setValue = function(value) {
				this.widget.currentRow[this.name] = value;
				handleCheckedState();
			};

			$scope.field.value = function(value) {
				if (typeof value !== "undefined") {
					this.widget.currentRow[this.name] = value;
					handleCheckedState();
					templateManager.updateTemplate();
				} else {
					return this.widget.currentRow[this.name];
				}
			};

			$scope.toggleValue = toggleValue;

			handleCheckedState();

			if (!input.is(':disabled')) {
				label.find(".zh-box").keypress(function(e) {
					e.preventDefault();
					toggleValue();
				});
			}
		}
	};
});

ZeedhiFields.directive('zhRadio', function() {
	return {
		link: function($scope, elem) {
			var field = $scope.field;

			if (!$scope.widget.currentRow[field.name] && !Util.isUndefinedOrNull(field.defaultOption)) {
				var option = field.dataSource.data[field.defaultOption];
				$scope.widget.currentRow[field.name] = option[field.valueField];
			}
		}
	};
});

ZeedhiFields.directive('zhSelect', function() {
	return {
		link: function($scope, element) {
			$scope.blurInput = function() {
				element.blur();
			};

			element.on('keydown', function(e) {
				return e.ctrlKey || e.keyCode == 9 || (e.keyCode >= 112 && e.keyCode <= 126); // TAB key
			});
		}
	};
});

ZeedhiFields.directive('zhNewSelect', function($timeout, ScreenService) {
	return {
		link: function($scope, element) {
			element.css('display', 'none');
			
			function buildOptions(ul) {
				ul.empty();

				var options = element.find('option');
				options.each(function(index, item) {
					item = $(item);
					var display = item.data('display');
					if (Util.isUndefined(item.attr('data-placeholder')) && Util.isUndefined(item.attr('placeholder'))) {
						var li = '<li class="option ' + (item.is(':selected') ? 'selected' : '') + '" data-value="' + item.val() + (display ?
							'" data-display="' + display : '') + '">' + item.text() + '</li>';
						$(li).appendTo(ul);
					}
				});
			}

			function getParentFixedTop(el) {
				var parents = el.parents();
				var top = 0;
				parents.each(function(idx, item) {
					item = $(item);
					if ((item.css('position') === "fixed" || item.css('position') === "absolute") &&
						item.parent() && (item.parent().css('position') === "fixed" || item.parent().css('position') === "absolute")) {
						top = item.offset().top;
						return false;
					}
				});

				return top;
			}

			if (!element.next().hasClass("new-select")) {
				element.after('<div class="new-select ' + (element.attr("class") || "") + (element.attr("disabled") ? "disabled" : '" tabindex="0') +
					'"></div>');
				var dropdown = element.next();

				dropdown.blur(function(evt) {
		            $scope.showLoading = false;
		        });			
	
				dropdown.focus(function(evt) {
		            $scope.showLoading = true;
		        });			
	
				$timeout(function() {
					if ($scope.field && $scope.field.isReadOnly()) {
						dropdown.attr('readonly', true);
					}

					dropdown[0].field = $scope.field;

					$scope.$on('$destroy', function() {
						delete dropdown[0].field;
					});

					if (element.attr('data-zh-focus-field')) {
						dropdown.focus();
					}

					var selected = element.find("option").filter(":selected");
					var selectedText = "";
					//The :selected query will return the first item if none is selected
					if (selected.attr("selected")) {
						selectedText = selected.data("display") || selected.text();
					}
					$('<span class="current">' + selectedText + '</span>').appendTo(dropdown);

					var ul = $('<ul class="list zh-new-select-list"></ul>').appendTo(dropdown);
					
					// prevent mouse events to propagate
					ul.on('mousedown.new_select mousewheel.new_select wheel.new_select DOMMouseScroll.new_select', function(event) {
						event.stopPropagation();
					});

					var options = element.find("option");
					options.each(function(index, item) {
						item = $(item);
						var display = item.data('display');
						if (item.attr('data-placeholder') === undefined && item.attr('placeholder') === undefined) {
							var li = '<li class="option ' + (item.is(':selected') ? 'selected' : '') + '" data-value="' + item.val() + (display ?
								'" data-display="' + display : '') + '">' + ScreenService.i18n(item.text()) + '</li>';
							$(li).appendTo(ul);
						}
					});
					$scope.$watch(function() {
						return element.html();
					}, function(val) {
						if (val) {
							buildOptions(ul);
						}
					});
				}, 0, false);
			}

			if ($scope.field) {
				$scope.$watch(function() {
					return $scope.field.value();
				}, function(newValue) {
					var dropdown = element.next();
					var selectedIndex = element[0].selectedIndex;

					dropdown.find('.selected').removeClass('selected');

					var option = dropdown.find('.option').eq(selectedIndex);
					option.addClass('selected');

					var text = option.data('display') || option.text();
					dropdown.find('.current').text(text);
				});

				$scope.$watch('field.isReadOnly()', function updateSelectNativeReadOnly(isReadOnly) {
					if (isReadOnly) {
						dropdown.attr('readonly', true);
					} else {
						dropdown.removeAttr('readonly');
					}
				});
			}

			if (!document.initNewSelect) {
				// Open/close
				$(document).on('click.new_select', '.new-select', function(event) {
					var dropdown = $(this);

					if ((dropdown[0].field && dropdown[0].field.isReadOnly()) || dropdown.is('[readonly]')) return;

					$('.new-select').not(dropdown).removeClass('open');
					dropdown.toggleClass('open');

					if (dropdown.hasClass('open')) {
						dropdown.find('.focus').removeClass('focus');
						dropdown.find('.selected').addClass('focus');
						dropdown.find('.list').width(dropdown.innerWidth());
						var dropdownOffset = dropdown.offset().top;
						var popupModal = $('.popup-modal');
						if(popupModal.length){
							dropdownOffset -= popupModal.offset().top;
						}
						dropdown.find('.list').css('top', dropdownOffset + dropdown.outerHeight() - getParentFixedTop(dropdown));
						if (dropdown.offset().top + dropdown.outerHeight() + dropdown.find('.list').outerHeight() + 10 > $(window).height()) {
							dropdown.find('.list').css('margin-top', -1 * (dropdown.outerHeight() + dropdown.find('.list').outerHeight() - 5));
						} else {
							dropdown.find('.list').css('margin-top', 0);
						}
					} else {
						dropdown.find('.list').width(0);
						dropdown.focus();
					}
				});

				// Close when clicking or scrolling outside
				$(document).on('mousedown.new_select mousewheel.new_select wheel.new_select DOMMouseScroll.new_select', function(event) {
					if ($(event.target).closest('.zh-new-select-list').length === 0) {
						$('.new-select').removeClass('open');
					}
				});

				// Close when window resize
				$(window).on('resize.new_select', function(event) {
					$('.new-select').removeClass('open');
				});

				// Option click
				$(document).on('click.new_select', '.new-select .option', function(event) {
					var option = $(this);
					var dropdown = option.closest('.new-select');

					if (dropdown[0].field && dropdown[0].field.isReadOnly()) return;

					dropdown.find('.selected').removeClass('selected');
					option.addClass('selected');

					var text = option.data('display') || option.text();
					dropdown.find('.current').text(text);

					dropdown.prev('select').val(option.data('value')).trigger('change');
				});

				// Keyboard events
				$(document).on('keydown.new_select', '.new-select', function(event) {

					var dropdown = $(this);
					var focused_option = $(dropdown.find('.focus') || dropdown.find('.list .option.selected'));

					if (dropdown[0].field && dropdown[0].field.isReadOnly()) return;

					// Space or Enter
					if (event.keyCode == 32 || event.keyCode == 13) {
						if (dropdown.hasClass('open')) {
							focused_option.trigger('click');
						} else {
							dropdown.trigger('click');
						}
						return false;
						// Down
					} else if (event.keyCode == 40) {
						if (!dropdown.hasClass('open')) {
							dropdown.trigger('click');
						} else {
							if (focused_option.next().length > 0) {
								dropdown.find('.focus').removeClass('focus');
								var next = focused_option.next();
								next.addClass('focus');
								if (next[0].offsetTop + next.height() > next.parent().height() + next.parent().scrollTop()) {
									next.parent()[0].scrollTop += next.height();
								}
							}
						}
						event.stopImmediatePropagation();
						return false;
						// Up
					} else if (event.keyCode == 38) {
						if (!dropdown.hasClass('open')) {
							dropdown.trigger('click');
						} else {
							if (focused_option.prev().length > 0) {
								dropdown.find('.focus').removeClass('focus');
								var prev = focused_option.prev();
								prev.addClass('focus');
								if (prev[0].offsetTop < prev.parent().scrollTop()) {
									prev.parent()[0].scrollTop -= prev.height();
								}
							}
						}
						event.stopImmediatePropagation();
						return false;
						// Esc
					} else if (event.keyCode == 27) {
						if (dropdown.hasClass('open')) {
							dropdown.trigger('click');
						}
						// Tab
					} else if (event.keyCode == 9) {
						if (dropdown.hasClass('open')) {
							return false;
						}
					}
				});


				document.initNewSelect = true;
			}
		}
	};
});

ZeedhiFields.directive('zhAsyncChange', function() {
	return {
		link: function($scope, elem) {
			$scope.change = function() {
				if (this.field && Util.isFunction(this.field.change)) {
					this.field.change();
				}
			};
		}
	};
});